import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { SignService } from '../../services/sign.service';

@Component({
    selector: 'app-login',
    templateUrl: './login.page.html',
    styleUrls: ['./login.page.css']
})
export class LoginPage implements OnInit {
    public form: FormGroup;

    constructor(
        private router: Router,
        private signService: SignService,
        private formBuilder: FormBuilder
    ) {
        this.form = this.formBuilder.group({
            Email: [null, [Validators.required, Validators.email]],
            Password: [null, Validators.required],
            Remember: [false]
        });
    }

    public ngOnInit(): void {
        if (localStorage.getItem('segmybill_email')) {
            this.form.controls['Remember'].setValue(true);
            this.form.controls['Email'].setValue(localStorage.getItem('segmybill_email'));
        }
    }
    
    public async signIn(): Promise<void> {
        try {
            let sign = await this.signService.signIn(this.form.value).toPromise();
            this.signService.setUserInStorage(sign.Response);

            if (this.form.value['Remember'] == true) {
                localStorage.setItem('segmybill_email', this.form.value.Email);
            } else {
                localStorage.removeItem('segmybill_email');
            }

            this.router.navigate(['/app/templates']);
        } catch (err) {
            throw err;
        }
    }

    public goTo(url: string) {
        this.router.navigate([url]);
    }

}