import { Injectable } from '@angular/core';
import { HttpInterceptor, HttpRequest, HttpHandler, HttpEvent, HttpErrorResponse } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { Router } from '@angular/router';
import { SignService } from './sign.service';

@Injectable({
    providedIn: 'root'
})
export class AuthInterceptorService implements HttpInterceptor {

    constructor(
        private router: Router,
        private signService: SignService
    ) { }

    intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        let token;

        if (localStorage.getItem('segmybill_token')) {
            token = localStorage.getItem('segmybill_token');
        }

        let request = req;

        request = req.clone({
            setHeaders: {
                Authorization: token ? `Bearer ${token}` : '',
            }
        });

        
        return next.handle(request).pipe(
            catchError((err: HttpErrorResponse) => {

                if (err.status === 401) {
                    this.signService.logout();
                    this.router.navigateByUrl('/login');
                }

                return throwError(err);

            })
        );
    }

}