import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { Observable, Subject } from "rxjs";
import { CoreService } from "./core.service";


@Injectable({ providedIn: 'root' })
export class SignService extends CoreService {
    private routeSignIn = '/Login';
    private routeSignUp = '/SignUp';
    private routeIsAdmin = '/UserRoleByToken';

    private logged = new Subject<any>();
    public getIsLogged$ = this.logged.asObservable();
    public token: string;
    public isAdmin: any;

    constructor(http: HttpClient) {
        super(http);
        this.getIsAdmin();
    }

    public signIn(body: any): Observable<any> {
        return this.post(this.routeSignIn, body, true);
    }

    public signUp(body: any): Observable<any> {
        return this.post(this.routeSignUp, body);
    }

    public getIsAdmin(): Observable<any> {
        return this.get(this.routeIsAdmin);
    }

    public setUserInStorage(token: string) {
        this.savingStorage(token);
        this.isLogged();
    }

    private savingStorage(token: string) {
        localStorage.setItem('segmybill_token', token);
        this.token = token;
    }

    /** Verify if user is logged */
    public isLogged() {
        return localStorage.getItem('segmybill_token') ? true : false;
    }

    /** Logout user and clean localstorage */
    public logout() {
        this.token = null;
        localStorage.removeItem('segmybill_token');
    }
}