import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { SignService } from 'src/app/core/services/sign.service';
import { ComunityService } from '../../services/comunity.service';

@Component({
    selector: 'app-comunity',
    templateUrl: './comunity.page.html',
    styleUrls: ['./comunity.page.css']
})
export class ComunityPage implements OnInit {
    public users: any[] = [];
    public filteredUsers: any[] = [];
    public form: FormGroup;
    public showFiltrar: boolean = false;
    public isAdmin: boolean = false;

    constructor(
        private comunityService: ComunityService,
        private signService: SignService,
        private formBuilder: FormBuilder
    ) {
        this.form = this.formBuilder.group({
            userName: null,
            userEmail: null,
            userRol: null
        });
    }

    public async ngOnInit(): Promise<void> {
        this.loadComunity();
        let role = await this.signService.getIsAdmin().toPromise();
        this.isAdmin = role?.Response == 'Admin';
    }

    public async loadComunity(): Promise<void> {
        try {
            this.filteredUsers = [];
            this.users = await this.comunityService.getUsers().toPromise();
            for (let user of this.users) {
                user.role = user.role == 0 ? 'Admin' : 'User';
            }

            Object.assign(this.filteredUsers, this.users);
        } catch (err) {
            throw err;
        }
    }

    public onCloseFiltrar(): void {
        this.showFiltrar = false;
        if (
            (this.form.value.userName == null || this.form.value.userName == '') &&
            (this.form.value.userEmail == null || this.form.value.userEmail == '') &&
            (this.form.value.userRol == null || this.form.value.userRol == '')
        ) {
            Object.assign(this.filteredUsers, this.users);
        } else {
            this.filteredUsers = this.users.filter((f) => {
                return (this.form.value.userName != '' && f.username.includes(this.form.value.userName)) ||
                    (this.form.value.userEmail != '' && f.email.includes(this.form.value.userEmail)) ||
                    (this.form.value.userRol != '' && f.role.includes(this.form.value.userRol));
            });
        }
    }

    public async onRemove(item: any): Promise<void> {
        try {
            await this.comunityService.removeUser(item.email).toPromise();
            this.loadComunity();
        } catch (err) {
            throw err;
        }
    }

}