import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { TemplatesService } from '../../services/templates.service';
import * as moment from 'moment';
import { SignService } from 'src/app/core/services/sign.service';
@Component({
    selector: 'app-templates',
    templateUrl: './templates.page.html',
    styleUrls: ['./templates.page.css']
})
export class TemplatesPage implements OnInit {
    public file: any;
    public template: any;
    public templates: any[] = [];
    public filteredTemplates: any[] = [];
    
    public showFiltrar: boolean = false;
    public showSelectedTemplate: boolean = false;
    public showNewTemplate: boolean = false;
    public showPreview: boolean = false;
    public isAdmin: boolean = false;

    public previewYml: string;
    public form: FormGroup;

    constructor(
        private templateService: TemplatesService,
        private signService: SignService,
        private formBuilder: FormBuilder
    ) {
        this.form = this.formBuilder.group({
            templateName: [null],
            templateDate: [null]
        });
    }

    public async ngOnInit(): Promise<void> {
        this.loadTemplates();
        let role = await this.signService.getIsAdmin().toPromise();
        this.isAdmin = role?.Response == 'Admin';
    }

    private async loadTemplates(): Promise<void> {
        try {
            this.file = null;
            this.filteredTemplates = [];
            this.template = null;
            this.templates = await this.templateService.getTemplates().toPromise();
            for (let t of this.templates) {
                t.dateOfCreation = moment(t.dateOfCreation).format('DD-MM-YYYY');
            }
            
            Object.assign(this.filteredTemplates, this.templates);
        } catch (err) {
            throw err;
        }
    }

    public async onRemove(item: any): Promise<void> {
        try {
            await this.templateService.removeTemplate(item.templateName).toPromise();
            this.loadTemplates();
        } catch (err) {
            throw err;
        }
    }

    public onPreview(item: any): void {
        this.showPreview = true;
        this.previewYml = decodeURIComponent(atob(item.value).split('').map(function(c) {
            return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
        }).join(''));
    }

    public onSelect(item: any): void {
        this.file = null;
        this.template = item;
        this.showSelectedTemplate = true;
    }

    public onCloseFiltrar(): void {
        this.showFiltrar = false;
        if (
            (this.form.value.templateName == null || this.form.value.templateName == '') &&
            (this.form.value.templateDate == null || this.form.value.templateDate == '')
        ) {
            Object.assign(this.filteredTemplates, this.templates);
        } else {
            this.filteredTemplates = this.templates.filter((f) => {
                return (f.templateName.includes(this.form.value.templateName)) ||
                    (f.dateOfCreation == moment(this.form.value.templateDate).format('DD-MM-YYYY'));
            });
        }
    }

    public async onCloseSelectTemplate(action: string): Promise<void> {
        try {
            if (action == 'download' && this.file != null && this.template != null) {
                let formData = new FormData();
                formData.append('Pdf', this.file);
                formData.append('Template', JSON.stringify(this.template));
                let json = await this.templateService.selectTemplate(formData).toPromise();

                let element = document.createElement('a');
                element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(JSON.stringify(json, null, 4)));
                element.setAttribute('download', 'invoice.json');
                element.style.display = 'none';
                document.body.appendChild(element);
                element.click();
                document.body.removeChild(element);
            }

            this.file = null;
            this.template = null;
            this.showSelectedTemplate = false;
        } catch (err) {
            this.file = null;
            this.template = null;
            this.showSelectedTemplate = false;
            throw err;
        }
    }

    public async onCloseNewTemplate(action: string): Promise<void> {
        try {
            let formData = new FormData();
            if (action == 'add' && this.file != null) {
                formData.append('TemplateValue', this.file);
                await this.templateService.addTemplate(formData).toPromise();
                this.loadTemplates();
            }

            this.file = null;
            this.showNewTemplate = false;
        } catch (err) {
            this.file = null;
            this.showNewTemplate = false;
            throw err;
        }
    }

    public onClosePreview(): void {
        this.showPreview = false;
    }

    public onFileDropped(files: any[]): void {
        this.file = files[0];
    }

    public formatBytes(bytes: number, decimals?: number): string {
        if (bytes === 0) {
            return '0 Bytes';
        }
        const k = 1024;
        const dm = decimals <= 0 ? 0 : decimals || 2;
        const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
        const i = Math.floor(Math.log(bytes) / Math.log(k));
        return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
    }

}