import json

from src.BillSegmentation import segmentation_main

from flask import Flask, jsonify, request
import yaml
import base64
import os

app = Flask(__name__)


@app.route('/ping')
def ping():
    return jsonify({"message": "Pong!"})


@app.route('/getJsonB64Template', methods=['POST'])
def getJsonB64Template():
    try:
        if request.files:
            pdf = request.files["Pdf"]
            if pdf.filename.endswith(".pdf"):
                pathPdf = os.path.join("src", "BillSegmentation", "segmentation", "bills", pdf.filename)
                pdf.save(pathPdf)
            else:
                return jsonify({"Code": 400, "Message": "The file must be a .pdf"}), 400
        else:
            return jsonify({"Code": 400, "Message": "You must send a .pdf in the request"}), 400

        jsonTemplateObject = json.loads(request.form.get("Template"))
        new_template = {
            "EmailCreator": jsonTemplateObject['creatorEmail'],
            "DateOfCreation": jsonTemplateObject['dateOfCreation'],
            "TemplateName": jsonTemplateObject['templateName'],
            "Value": jsonTemplateObject['value']
        }

        b64TemplateValue = new_template.get("Value")
        stringTemplateValue = base64.b64decode(b64TemplateValue).decode('utf-8')
        yamlTemplate = yaml.safe_load(stringTemplateValue)

        textDict = {"data": yamlTemplate}

        pathTemplate = os.path.join("src", "BillSegmentation", "segmentation", "template",
                                    new_template.get("TemplateName") + ".yml")

        with open(pathTemplate, 'w') as outfile:
            yaml.dump(textDict.get("data"), stream=outfile, allow_unicode=True)

        # print('yaml dump: \n' + open(pathTemplate).read()),

        args = [pathPdf]
        parser = segmentation_main.create_parser()
        argsDef = parser.parse_args(args)
        segmentation_main.main(argsDef)

        os.remove(pathPdf)
        os.remove(pathTemplate)

        dirname = os.path.dirname(__file__)
        jsonPath = os.path.join(dirname, 'segmentatedBill.json')

        f = open(jsonPath, encoding="utf8")
        data = json.load(f)

        return jsonify(data), 200
    except Exception as ex:

        return jsonify({"Code": 500, "Message": ex.args[0]}), 500


@app.route('/getJsonByTemplate', methods=['POST'])
def getJsonByTemplate():
    try:
        if request.files:
            pdf = request.files["Pdf"]
            template = request.files["Template"]
            if pdf.filename.endswith(".pdf"):
                pathPdf = os.path.join("src", "BillSegmentation", "segmentation", "bills", pdf.filename)
                pdf.save(pathPdf)
            else:
                return jsonify({"Code": 400, "Message": " {Pdf: } The file must be a .pdf"}), 400

            if template.filename.endswith(".yml"):
                pathTemplate = os.path.join("src", "BillSegmentation", "segmentation", "template", template.filename)
                template.save(pathTemplate)
            else:
                return jsonify({"Code": 400, "Message": " {Template: } The file must be a .yml"}), 400

        else:
            return jsonify({"Code": 400, "Message": "You must send a .pdf and a .yml in the request"}), 400

        args = [pathPdf]
        parser = segmentation_main.create_parser()
        argsDef = parser.parse_args(args)
        segmentation_main.main(argsDef)

        os.remove(pathPdf)
        os.remove(pathTemplate)

        dirname = os.path.dirname(__file__)
        jsonPath = os.path.join(dirname, 'segmentatedBill.json')

        f = open(jsonPath, encoding="utf8")
        data = json.load(f)

        return jsonify(data), 200

    except Exception as ex:
        return jsonify({"Code": 500, "Message": ex.args[0]}), 500


if __name__ == '__main__':
    app.run(debug=True, port=4000)
