import os
import yaml
import pkg_resources
from collections import OrderedDict
from .bill_template import BillTemplate
import codecs
import chardet


def map_ordered_dict(stream):
    # S'encarrega del mapeig del yml a tipus OrderedDict a partir dels loaders de la llibreria yaml

    class OrderedLoader(yaml.Loader):
        pass

    def map_constructor(loader, node):
        loader.flatten_mapping(node)
        return OrderedDict(loader.construct_pairs(node))

    OrderedLoader.add_constructor(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, map_constructor
    )

    loaded_yaml = yaml.load(stream, OrderedLoader)
    return loaded_yaml


def read_templates():
    # Fa la carrega del template que li hem afegit a la carpeta template en la crida de la api

    template_path = pkg_resources.resource_filename(__name__, "template")

    # Extreu el nom del document de la carpeta
    for path, subdirs, files in os.walk(template_path):
        file_name = files.pop(0)
        if file_name.endswith(".yml"):
            # Optenim el file per extreure el seu stream per fer el mapping del contingut de la template
            with open(os.path.join(path, file_name), "rb") as f:
                encoding = chardet.detect(f.read())["encoding"]
            with codecs.open(
                    os.path.join(path, file_name), encoding=encoding
            ) as template_file:
                try:
                    # Fem el mapping de la template
                    tmplCollection = map_ordered_dict(template_file.read())
                except yaml.parser.ParserError:
                    # Ha fallat al fer el mapping
                    continue

            tmplCollection["template_name"] = file_name

            # Comprova que dins de la plantilla hi hagi la key must_to_words
            assert "must_to_words" in tmplCollection.keys(), "Falta parametre must_to_words de la taula."

            # Tenint en compte que pot ser una llista o no, forço tractar-la com una
            if type(tmplCollection["must_to_words"]) is not list:
                tmplCollection["must_to_words"] = [tmplCollection["must_to_words"]]

            # Generem una template segons el model i l'afegim a la llista
            return BillTemplate(tmplCollection)

    # TODO: THROW EXCEPTION No s'ha trobat cap template
    return None
