import re

DEFAULT_LINE_SEPARATOR = {"line_separator": r"\n"}


def parse(bill_template, field_name, lines_configuration, bill_string):
    # Extreu les linies formades a la plantilla de dins la factura

    actual_line = {}
    line_extraction = []
    # S'extreu el contingut de la plantilla
    line_config = DEFAULT_LINE_SEPARATOR.copy()
    line_config.update(lines_configuration)

    # Perque hi hagi lineas han de contenir els camps begin, finish, line
    assert "begin" in line_config, "Falta parametre begin de la linia"
    assert "finish" in line_config, "Falta parametre finish de la linia"
    assert "line" in line_config, "Falta parametre line de la linia"

    # Extreu a través del string buscant una coincidencia al patró que marca el camp,
    # retornant un objecte tipus Match i en cas contrari un None.
    finish = re.search(line_config["finish"], bill_string)
    begin = re.search(line_config["begin"], bill_string)

    if not finish or not begin:
        # TODO THROW EXCEPTION: No s'ha troba inici o final de linia
        return
    # Retalla la part de les linies de principi al final marcat
    bill_string = bill_string[begin.end(): finish.start()]

    # En cas de no tenir indicada si es la primera fila i la ultima asignarà a la linia definida com a primera i unica
    if "last_line" not in line_config and "first_line" not in line_config:
        line_config["first_line"] = line_config["line"]

    primera_fila = False
    for line in re.split(line_config["line_separator"], bill_string):
        #  Si conté linies buides passem a la seguent
        if not line.strip("").strip("\n").strip("\r") or not line:
            continue
        if "first_line" in line_config:
            # Mira si la linia actual compleix amb el patró
            coincidence = re.search(line_config["first_line"], line)
            if coincidence:
                # En cas d'entrar i que actual_line sigui empty, serà la primera linia trobada
                if actual_line:
                    line_extraction.append(actual_line)
                # A partir de d'una list comprension extreiem els matchs fets amb el seu camp i el seu valor
                actual_line = {field: value.strip() if value else "" for field, value in coincidence.groupdict().items()}
                # Ja s'ha trobat la primera linia, aixo fa que es posi el bolea a true i pugui passar a trobar l'ultima
                # l'ultima linia
                primera_fila = True
                continue
        # En cas que no s'hagi trobat encara la el patró de la primera linia segueix al seguent
        if primera_fila is False:
            continue
        # Si es troba l'ultima linea es fa el mateix procediment que la primera pero s'afegeix directament a la llista
        # i nega el bool de fist_line_found
        if "last_line" in line_config:
            coincidence = re.search(line_config["last_line"], line)
            if coincidence:
                actual_line = parse_actual_line(coincidence, actual_line)
                if actual_line:
                    line_extraction.append(actual_line)
                actual_line = {}

                primera_fila = False
                continue
        # En cas que volguem saltar alguna linea en concret s'ha de marcar a la template com a file de lines
        if "skip_line" in line_config:
            # forço la que es contingui dins d'una llista
            # per passar al seguent minim ha de tenir un coincident dins d'aquesta
            if isinstance(line_config["skip_line"], list):
                skip_line_content = [re.search(x, line) for x in line_config["skip_line"]]
            else:
                skip_line_content = [re.search(line_config["skip_line"], line)]
            if any(skip_line_content):
                # Passem de linia
                continue
        # Ja que hi ha 4 tipus de linea i no ha entrat a cap aquesta es el cas de la linia normal
        coincidence = re.search(line_config["line"], line)
        if coincidence:
            # Aquesta linia esta entre la de inici i la de final, i l'agrega a la actual_line
            actual_line = parse_actual_line(coincidence, actual_line)
            continue
        # Si no hi ha coincidencia la linia s'ignora

    # Ja s'han tret totes les linies
    if actual_line:
        line_extraction.append(actual_line)

    # Es es busca la variable asignada i es passa al tipus de valor que es vol mostrar ja que per defecte es un string
    types = line_config.get("types", [])
    for x in line_extraction:
        for subfield in x.keys():
            if subfield in types:
                x[subfield] = bill_template.to_type(x[subfield], types[subfield])

    return line_extraction


def parse_actual_line(coincidence, actual_line):
    # Serveix per parsejar la fila a on estas abans d'agragarla al definitiu de linies
    for field, value in coincidence.groupdict().items():
        actual_line[field] = "%s%s%s" % (
            actual_line.get(field, ""),
            actual_line.get(field, "") and "\n" or "",
            value.strip() if value else "",
        )
    return actual_line
